/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockOperation;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.support.UISupport;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.support.ADialogBuilder;
import com.eviware.x.form.support.AField;
import com.eviware.x.form.support.AForm;
import com.eviware.x.form.support.AField.AFieldType;

public class GenerateMockServiceAction extends AbstractSwingAction<WsdlInterface>
{
	private static final String CREATE_MOCKSUITE_OPTION = "<create>";
//	private XFormDialog dialog;

	public GenerateMockServiceAction( WsdlInterface iface )
	{
		super( "Generate MockService", "Generates a MockService containing all Operations in this Interface", iface );
	}

	public void actionPerformed( ActionEvent e, WsdlInterface iface )
	{
// Cannot (?) store and reuse widgets on SWT.
//		if( dialog == null )
      XFormDialog dialog = ADialogBuilder.buildDialog( Form.class );
		
		WsdlProject project = ( WsdlProject ) iface.getProject();
		String[] mockServices = ModelSupport.getNames( project.getMockServices(), new String[] {CREATE_MOCKSUITE_OPTION});
		dialog.setOptions( Form.MOCKSERVICE, mockServices );
		
		dialog.setValue( Form.PATH, "/mock" + iface.getName() );
		dialog.setValue( Form.PORT, "8088" );
		
		if( dialog.show() )
		{
			String mockServiceName = dialog.getValue( Form.MOCKSERVICE );
			WsdlMockService mockService = ( WsdlMockService ) project.getMockServiceByName( mockServiceName );
			
			if( mockService == null || mockServiceName.equals( CREATE_MOCKSUITE_OPTION ))
			{
				mockServiceName = UISupport.prompt( "Specify name of MockService to create", getName(), iface.getName() + " MockService" );
				if( mockServiceName == null )
					return;
				
				mockService = ( WsdlMockService ) project.addNewMockService( mockServiceName );
			}
			
			mockService.setPath( dialog.getValue( Form.PATH ) );
			try
			{
				mockService.setPort( Integer.parseInt( dialog.getValue( Form.PORT )) );
			}
			catch( NumberFormatException e1 )
			{
			}
			
			for( int i = 0; i < iface.getOperationCount(); i++ )
			{
				WsdlOperation operation = ( WsdlOperation ) iface.getOperationAt( i );
				WsdlMockOperation mockOperation = ( WsdlMockOperation ) mockService.addNewMockOperation( operation );
				mockOperation.addNewMockResponse( "Response 1", true );
			}
			
			UISupport.showDesktopPanel( mockService );
		}
	}
	
	@AForm( name="Generate MockService", 
				description = "Set options for generated MockOperations for this Interface")
	private interface Form 
	{
		@AField(name = "MockService", description = "The MockService to create or use", type = AFieldType.ENUMERATION )
		public final static String MOCKSERVICE = "MockService";
		
		@AField(name = "Path", description = "The URL path to mount on", type = AFieldType.STRING )
		public final static String PATH = "Path";

		@AField(name = "Port", description = "The endpoint port to listen on", type = AFieldType.STRING )
		public final static String PORT = "Port";
	}
}
