/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.loadtest;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.loadtest.WsdlLoadTest;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormFactory;
import com.eviware.x.form.XForm.FieldType;

/**
 * Displays the LoadTest Options dialog
 * 
 * @author Ole.Matzura
 */

public class LoadTestOptionsAction extends AbstractAction
{
	private static final String THREAD_STARTUP_DELAY = "Thread startup delay";
	private static final String RESET_STATISTICS = "Reset statistics";
	private static final String CALC_TPS = "Calculate TPS/BPS";
	private static final String INCLUDE_REQUEST = "Include request write";
	private static final String INCLUDE_RESPONSE = "Include response read";
	private static final String CLOSE_CONNECTIONS = "Close connections";
	private static final String SAMPLE_INTERVAL = "Sample interval";
	private final WsdlLoadTest loadTest;
	private XFormDialog dialog;

	public LoadTestOptionsAction( WsdlLoadTest loadTest )
   {
      this.loadTest = loadTest;
      
		putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/options.gif"));
      putValue( Action.SHORT_DESCRIPTION, "Sets options for this LoadTest" );
   }

	public void actionPerformed(ActionEvent e)
	{
		if( dialog == null )
			buildDialog();
		
		StringToStringMap values = new StringToStringMap();
		
		values.put( THREAD_STARTUP_DELAY, String.valueOf( loadTest.getStartDelay()) );
		values.put( RESET_STATISTICS, String.valueOf( loadTest.getResetStatisticsOnThreadCountChange()) );
		values.put( CALC_TPS, String.valueOf( loadTest.getCalculateTPSOnTimePassed()) );
		values.put( SAMPLE_INTERVAL, String.valueOf( loadTest.getSampleInterval()) );
		
		Settings settings = loadTest.getSettings();
		values.put( INCLUDE_REQUEST, String.valueOf( settings.getBoolean( HttpSettings.INCLUDE_REQUEST_IN_TIME_TAKEN )));
		values.put( INCLUDE_RESPONSE, String.valueOf( settings.getBoolean( HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN )));
		values.put( CLOSE_CONNECTIONS, String.valueOf( settings.getBoolean( HttpSettings.CLOSE_CONNECTIONS )));
	
		values = dialog.show( values );
		
		if( dialog.getReturnValue() == XFormDialog.OK_OPTION )
		{
			try
			{
				loadTest.setStartDelay(Integer.parseInt(values.get(THREAD_STARTUP_DELAY)));
				loadTest.setResetStatisticsOnThreadCountChange( Boolean.parseBoolean( values.get( RESET_STATISTICS )));
				loadTest.setCalculateTPSOnTimePassed( Boolean.parseBoolean( values.get( CALC_TPS )));
				loadTest.setSampleInterval( Integer.parseInt(values.get(SAMPLE_INTERVAL)));
				settings.setBoolean( HttpSettings.INCLUDE_REQUEST_IN_TIME_TAKEN, 
						Boolean.parseBoolean( values.get( INCLUDE_REQUEST )));
				settings.setBoolean( HttpSettings.INCLUDE_RESPONSE_IN_TIME_TAKEN, 
						Boolean.parseBoolean( values.get( INCLUDE_RESPONSE )));
				settings.setBoolean( HttpSettings.CLOSE_CONNECTIONS, 
						Boolean.parseBoolean( values.get( CLOSE_CONNECTIONS )));
			}
			catch (NumberFormatException ex)
			{
			}			
		}
	}

	private void buildDialog()
	{
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder( "LoadTest Options" );
	   XForm form = builder.createForm( "Basic" );
		
	   form.addTextField( THREAD_STARTUP_DELAY, "The delay before starting a thread in ms", FieldType.TEXT ).setWidth( 10 );
	   form.addCheckBox( RESET_STATISTICS, "when the number of threads changes" );
	   form.addCheckBox( CALC_TPS, "based on actual time passed" );
	   form.addCheckBox( INCLUDE_REQUEST, "in calculated time" );
	   form.addCheckBox( INCLUDE_RESPONSE, "in calculated time" );
	   form.addCheckBox( CLOSE_CONNECTIONS, "after each request" );
	   form.addTextField( SAMPLE_INTERVAL, "the frequency of statistics updates in ms", FieldType.TEXT ).setWidth( 10 );
	   
	   dialog = builder.buildDialog( builder.buildOkCancelHelpActions( HelpUrls.LOADTESTOPTIONS_HELP_URL ), 
	   		"Specify general options for this LoadTest", UISupport.TOOL_ICON );
	}
}