/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.project;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.support.UISupport;

/**
 * Removes a WsdlProject from the workspace
 * 
 * @author Ole.Matzura
 */

public class RemoveProjectAction extends AbstractAction
{
   private final WsdlProject project;

	public RemoveProjectAction( WsdlProject project)
   {
      super( "Remove" );
		this.project = project;
      putValue( Action.SHORT_DESCRIPTION, "Removes this project from the workspace" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "DELETE" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	if( hasRunningTests())
   	{
   		UISupport.showErrorMessage( "Cannot remove Interface due to running tests" );
   		return;
   	}
   	
   	Boolean retval = UISupport.confirmOrCancel( "Save project [" + project.getName() + "] before removing?", "Remove Project" );
   	if( retval == null )
   		return;
   	
   	if( retval.booleanValue() )
      {
         try
			{
				project.save();
			}
			catch( IOException e1 )
			{
				UISupport.showErrorMessage( e1 );
			}
      }

      project.getWorkspace().removeProject( project );
   }

	private boolean hasRunningTests()
	{
		for( int c = 0; c < project.getTestSuiteCount(); c++ )
		{
			TestSuite testSuite = project.getTestSuiteAt( c );
			for( int i = 0; i < testSuite.getTestCaseCount(); i++ )
			{
				if( SoapUI.getTestMonitor().hasRunningTest( testSuite.getTestCaseAt( i )))
				{
					return true;
				}
			}
		}

		return false;
	}
}
