/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.testcase;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.support.UISupport;

/**
 * Clones a WsdlTestCase
 * 
 * @author Ole.Matzura
 */

public class CloneTestCaseAction extends AbstractAction
{
	private final WsdlTestCase testCase;

	public CloneTestCaseAction( WsdlTestCase testCase ) 
   {
      super( "Clone TestCase" );
		this.testCase = testCase;
      putValue( Action.SHORT_DESCRIPTION, "Clones this TestCase" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "F9" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      String name = UISupport.prompt( "Specify name of cloned TestCase", "Clone TestCase", "Copy of " + testCase.getName() );
      if( name == null ) return;
      
      Project project = testCase.getTestSuite().getProject();
      if( project.getTestSuiteCount() > 1 )
      {
      	// ask which testsuite to clone to
      	List<String> names = new ArrayList<String>();
      	for( int c = 0; c < project.getTestSuiteCount(); c++ )
      	{
      		names.add( project.getTestSuiteAt( c ).getName() );
      	}
      	
      	String testSuite = (String) UISupport.prompt( "Select target TestSuite", "Clone TestCase", names.toArray(), 
      			testCase.getTestSuite().getName() );
      	
      	if( testSuite == null )
      		return;
      	
      	WsdlTestSuite targetSuite = (WsdlTestSuite) project.getTestSuiteAt( names.indexOf( testSuite ));
		   WsdlTestCase newTestCase = targetSuite.cloneTestCase( testCase, name );
		   SoapUI.selectModelItem( newTestCase );
      }
      else
      {
		   WsdlTestCase newTestCase = ((WsdlTestSuite)testCase.getTestSuite()).cloneTestCase( testCase, name );
		   SoapUI.selectModelItem( newTestCase );
      }
   }
}
