/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.mock;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;

import javax.servlet.http.HttpServletResponse;

import org.mortbay.jetty.HttpFields;

import com.eviware.soapui.impl.wsdl.panels.mockoperation.WsdlMockResultMessageExchange;
import com.eviware.soapui.impl.wsdl.teststeps.actions.ShowMessageExchangeAction;
import com.eviware.soapui.model.mock.MockResult;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.DefaultActionList;
import com.eviware.soapui.support.types.StringToStringMap;

public class WsdlMockResult implements MockResult
{
	private WsdlMockResponse mockResponse;
	private String responseContent;
	private long timeTaken;
	private long timestamp;
	private DefaultActionList actions;
	private StringToStringMap responseHeaders = new StringToStringMap();
	private WsdlMockRequest mockRequest;
	private HttpServletResponse response;

	public WsdlMockResult( WsdlMockRequest request, HttpServletResponse response ) throws Exception
	{
		this.response = response;
		timestamp = System.currentTimeMillis();
		mockRequest = request;
	}

	public WsdlMockRequest getMockRequest()
	{
		return mockRequest;
	}

	public ActionList getActions()
	{
		if( actions == null )
		{
			actions = new DefaultActionList( "MockResult" );
			actions.setDefaultAction( new ShowMessageExchangeAction( new WsdlMockResultMessageExchange( this ), "MockResult") );
		}
		
		return actions;
	}

	public WsdlMockResponse getMockResponse()
	{
		return mockResponse;
	}
	
	public String getResponseContent()
	{
		return responseContent;
	}
	
	public long getTimeTaken()
	{
		return timeTaken;
	}

	public long getTimestamp()
	{
		return timestamp;
	}

	public void setTimestamp( long timestamp )
	{
		this.timestamp = timestamp;
	}

	public void setTimeTaken( long timeTaken )
	{
		this.timeTaken = timeTaken;
	}
	
	public StringToStringMap getResponseHeaders()
	{
		return responseHeaders;
	}

	public void setMockResponse( WsdlMockResponse mockResponse )
	{
		this.mockResponse = mockResponse;
	}

	public void setReponseContent( String responseContent )
	{
		this.responseContent = responseContent;
	}
	
	public void finish()
	{
		HttpFields httpFields = ((org.mortbay.jetty.Response)response).getHttpFields();
		
		Enumeration<String> e = httpFields.getFieldNames();
		while( e.hasMoreElements() )
		{
			String nextElement = e.nextElement();
			responseHeaders.put( nextElement, httpFields.getStringField( nextElement ) );
		}
		
		response = null;
	}

	public void addHeader( String name, String value )
	{
		if( response != null )
			response.addHeader( name, value );
		else
			responseHeaders.put( name, value );
	}

	public boolean isCommitted()
	{
		return response.isCommitted();
	}

	public void setContentType( String string )
	{
		response.setContentType( string );
	}

	public OutputStream getOutputStream() throws IOException
	{
		return response.getOutputStream();
	}

	public void initResponse()
	{
		response.setStatus( HttpServletResponse.SC_OK );
	}

	public boolean isDiscarded()
	{
		return false;
	}
}
