/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.attachments;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;

import javax.swing.table.AbstractTableModel;

import com.eviware.soapui.impl.wsdl.mock.WsdlMockResponse;
import com.eviware.soapui.impl.wsdl.support.WsdlAttachment;
import com.eviware.soapui.model.iface.Attachment;

/**
 *
 * @author emibre
 */
public class MockAttachmentTableModel extends AbstractTableModel implements PropertyChangeListener, AttachmentTableModel {
	
	private boolean isResponse = false;
	private WsdlMockResponse mockResponse;
	
	/** Creates a new instance of AttachmentTableModel */
	public MockAttachmentTableModel(WsdlMockResponse mockOperation2, boolean isResponse) {
		this.mockResponse = mockOperation2;
		this.isResponse = isResponse;
		
		mockResponse.addPropertyChangeListener( WsdlMockResponse.MOCKRESULT_PROPERTY, this );
		mockResponse.addPropertyChangeListener( WsdlMockResponse.ATTACHMENTS_PROPERTY, this );
	}
	
	public void release()
	{
		mockResponse.removePropertyChangeListener( this );
	}
	
	public void addFile( File file, boolean cacheInRequest) throws IOException 
	{
		if (isResponse)
		{
			Attachment attachment = mockResponse.attachFile( file, cacheInRequest );
			attachment.setContentType( ContentTypeHandler.getContentTypeFromFilename(file.getName()));
			this.fireTableRowsInserted(mockResponse.getAttachmentCount(), mockResponse.getAttachmentCount());
		}
	}
		
	public void removeAttachment(int[] rowIndexes) {
		Arrays.sort(rowIndexes);
		for (int i = rowIndexes.length-1; i >=0; i--)
			removeAttachment(rowIndexes[i]);
	}
	
	public void removeAttachment(int rowIndex) {
		if (isResponse)
		{
			mockResponse.removeAttachment(mockResponse.getAttachmentAt(rowIndex));
			this.fireTableRowsDeleted(rowIndex, rowIndex);
		}
	}
	
	public int getRowCount() {
		if (isResponse)
			return mockResponse.getAttachmentCount();
		else {
			try {
				// Response may not exist yet and attachments may be null
				return mockResponse.getMockResult().getMockRequest().getRequestAttachments().length;
			} catch (Exception e) {
				return 0;
			}
		}
	}
	
	public int getColumnCount() {
		return isResponse ? 6 : 5;
	}
	
	public Attachment getAttachmentAt( int rowIndex )
	{
		if (isResponse)
			return mockResponse.getAttachmentAt(rowIndex);
		else
			return mockResponse.getMockResult().getMockRequest().getRequestAttachments()[rowIndex];
	}
	
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (rowIndex > getRowCount())
			return null;
		
		Attachment att = null;
		if (isResponse)
			att = mockResponse.getAttachmentAt(rowIndex);
		else
			att = mockResponse.getMockResult().getMockRequest().getRequestAttachments()[rowIndex];
		
		switch (columnIndex) {
			case 0:
				return att.getName();
			case 1:
				return att.getContentType();
			case 2:
				return att.getSize();
			case 3:
				return att.getPart();
			case 4:
				return att.getAttachmentType();
			case 5:
				return att.getContentID();
			default:
				return null;
		}
	}
	
	public int findColumn(String columnName) {
		if (columnName.equals("Name"))
			return 0;
		else if (columnName.equals("Content type"))
			return 1;
		else if (columnName.equals("Size"))
			return 2;
		else if (columnName.equals("Part"))
			return 3;
		else if (columnName.equals("Type"))
			return 4;
		
		return -1;
	}
	
	public String getColumnName(int column) {
		if (column == 0)
			return "Name";
		else if (column == 1)
			return "Content type";
		else if (column == 2)
			return "Size";
		else if (column == 3)
			return "Part";
		else if (column == 4)
			return "Type";
		else if (column == 5)
			return "ContentID";
		else
			return null;
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return isResponse && (columnIndex == 1 || columnIndex == 3 || columnIndex == 5 );
	}
	
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		if( !isResponse )
			return;
		
		WsdlAttachment att = ( WsdlAttachment ) mockResponse.getAttachmentAt(rowIndex);
		if( columnIndex == 1 )
			att.setContentType( (String)aValue );
		else if( columnIndex == 3 )
			att.setPart( (String)aValue );
		else if( columnIndex == 5 )
			att.setContentID( (String)aValue );
		
		fireTableRowsUpdated( rowIndex, rowIndex );
	}

	/**
	 * Update table when attachments or response changes
	 */
	
	public void propertyChange(PropertyChangeEvent evt)
	{
		fireTableDataChanged();
	}
	
}
