/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.support.TestRunComponentEnabler;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.impl.wsdl.teststeps.PropertyTransfer;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepListener;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep.ValueTransferResult;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepProperty;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JUndoableTextArea;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.support.xml.XmlUtils;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for TransferResponseValuesTestStep
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesDesktopPanel extends JPanel implements DesktopPanel
{
	private final TransferResponseValuesTestStep transferStep;
	private DefaultListModel listModel;
	private JList transferList;
	private JTextArea sourceArea;
	private JTextArea targetArea;
	private JButton copyButton;
	private JButton deleteButton;
	private JButton declareButton;
	private JComboBox sourcePropertyCombo;
	private JComboBox targetPropertyCombo;
	private JComboBox sourceStepCombo;
	private JComboBox targetStepCombo;
	private DefaultComboBoxModel sourceStepModel;
	private DefaultComboBoxModel targetStepModel;
	private TestStepNameListener sourceTestStepNameListener;
	private TestStepNameListener targetStepNameListener;
	private TestStepPropertiesListener sourceStepPropertiesListener;
	private TestStepPropertiesListener targetStepPropertiesListener;
	private TransferPropertyChangeListener transferPropertyChangeListener = new TransferPropertyChangeListener();
	private boolean selecting;
	private InternalTestSuiteListener testSuiteListener;
	private TestRunComponentEnabler componentEnabler;
	private JCheckBox failTransferCheckBox;
	private JButton runButton;
	private JButton renameButton;
	private JCheckBox setNullCheckBox;
	private JCheckBox transferTextContentCheckBox;
	private JCheckBox ignoreEmptyCheckBox;
	private JCheckBox transferAllCheckBox;

	public TransferResponseValuesDesktopPanel(TransferResponseValuesTestStep testStep)
	{
		super( new BorderLayout() );
		this.transferStep = testStep;
		componentEnabler = new TestRunComponentEnabler( testStep.getTestCase() );
		
		buildUI();
		
		testSuiteListener = new InternalTestSuiteListener();
		transferStep.getTestCase().getTestSuite().addTestSuiteListener( testSuiteListener );
	}

	protected void buildUI()
	{
		JSplitPane splitPane = UISupport.createHorizontalSplit();
		
		listModel = new DefaultListModel();
		
		for( int c = 0; c < transferStep.getTransferCount(); c++ )
		{
			listModel.addElement( transferStep.getTransferAt( c  ).getName() );
		}
		
		transferList = new JList( listModel );
		transferList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		transferList.addListSelectionListener( new TransferListSelectionListener());
		componentEnabler.add( transferList );
		
		JScrollPane listScrollPane = new JScrollPane( transferList );
		listScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Transfers" ), 
				BorderFactory.createEmptyBorder()));
		
		splitPane.setLeftComponent( listScrollPane);
		
		JSplitPane innerSplit = UISupport.createVerticalSplit();
		innerSplit.setBorder( null );
		sourceArea = new JUndoableTextArea();
		sourceArea.setToolTipText( "XPath selection from source property" );
		sourceArea.setEnabled( false );
		sourceArea.getDocument().addDocumentListener( new SourceAreaDocumentListener());
		componentEnabler.add( sourceArea );
		
		targetArea = new JUndoableTextArea();
		targetArea.setToolTipText( "XPath target in target property" );
		targetArea.setEnabled( false );
		targetArea.getDocument().addDocumentListener( new TargetAreaDocumentListener());
		componentEnabler.add( targetArea );
		
		JPanel sourcePanel = new JPanel( new BorderLayout() );
		sourcePanel.add( new JScrollPane( sourceArea ), BorderLayout.CENTER );
		JXToolBar toolbar = createSourceToolbar();
		sourcePanel.add( toolbar, BorderLayout.NORTH );
		sourcePanel.setBorder( BorderFactory.createEmptyBorder( 0, 3, 3, 3 ));
		
		innerSplit.setTopComponent( sourcePanel );
		
		JPanel targetPanel = new JPanel( new BorderLayout() );
		targetPanel.add( new JScrollPane( targetArea ), BorderLayout.CENTER );
		toolbar = createTargetToolbar();
		targetPanel.add( toolbar, BorderLayout.NORTH );
		targetPanel.setBorder( BorderFactory.createEmptyBorder( 0, 3, 3, 3 ));
		
		innerSplit.setBottomComponent( targetPanel );
		
		innerSplit.setResizeWeight( 0.5 );
		innerSplit.setDividerLocation( 0.5 );
		
		JPanel panel = createTransferOptions();
		
		JPanel innerPanel = new JPanel( new BorderLayout() );
		innerPanel.add( innerSplit, BorderLayout.CENTER );
		innerPanel.add( panel, BorderLayout.SOUTH );
		
		splitPane.setRightComponent( innerPanel );
		splitPane.setResizeWeight( 0.1 );
		splitPane.setDividerLocation( 120 );
		
		add( splitPane, BorderLayout.CENTER );
		
		ButtonBarBuilder builder = createButtonBar();
		
		add( builder.getPanel(), BorderLayout.SOUTH );
		
		setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		setPreferredSize( new Dimension( 550, 400 ));
		
		if( listModel.getSize() > 0 ) 
			transferList.setSelectedIndex( 0 );
		
		componentEnabler.add( deleteButton );
		componentEnabler.add( declareButton );
		componentEnabler.add( runButton );
		componentEnabler.add( copyButton );
		componentEnabler.add( renameButton );
		componentEnabler.add( failTransferCheckBox );
		componentEnabler.add( setNullCheckBox );
		componentEnabler.add( transferTextContentCheckBox );
		componentEnabler.add( ignoreEmptyCheckBox );
		componentEnabler.add( transferAllCheckBox );
	}

	protected ButtonBarBuilder createButtonBar()
	{
		ButtonBarBuilder builder = new ButtonBarBuilder();
		builder.addFixed( new JButton( new AddAction() ) );
		builder.addRelatedGap();
		copyButton = new JButton( new CopyAction() );
		copyButton.setEnabled( false );
		builder.addFixed( copyButton);
		builder.addRelatedGap();
		renameButton = new JButton( new RenameAction() );
		renameButton.setEnabled( false );
		builder.addFixed( renameButton);
		builder.addRelatedGap();
		deleteButton = new JButton( new DeleteAction() );
		deleteButton.setEnabled( false );
		builder.addFixed( deleteButton);
		builder.addRelatedGap();
		declareButton = new JButton( new DeclareNamespacesAction() );
		declareButton.setEnabled( false );
		builder.addFixed( declareButton);
		builder.addRelatedGap();
		runButton = new JButton( new RunAction() );
		runButton.setEnabled( transferStep.getTransferCount() > 0 );
		builder.addFixed( runButton);
		builder.addGlue();
		JButton closeButton = new JButton( new CloseAction() );
		builder.addFixed( closeButton);
		builder.addRelatedGap();
		builder.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.TRANSFERSTEPEDITOR_HELP_URL )));
		builder.setBorder( 
				BorderFactory.createCompoundBorder( 
						BorderFactory.createCompoundBorder( 
								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.GRAY ),
								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.WHITE )),
						BorderFactory.createEmptyBorder( 3, 0, 0, 0 )));
		return builder;
	}

	protected JPanel createTransferOptions()
	{
		JPanel panel = new JPanel( new GridLayout( 3, 2 ));
		failTransferCheckBox = new JCheckBox( "Fail transfer on error", false );
		failTransferCheckBox.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				PropertyTransfer currentTransfer = getCurrentTransfer();
				if( currentTransfer != null )
				{
					currentTransfer.setFailOnError( failTransferCheckBox.isSelected() );
				}
			}} );
		
		setNullCheckBox = new JCheckBox( "Set null on missing source", false );
		setNullCheckBox.setToolTipText( "Will set target to null if source is missing or null" );
		setNullCheckBox.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				PropertyTransfer currentTransfer = getCurrentTransfer();
				if( currentTransfer != null )
				{
					currentTransfer.setSetNullOnMissingSource( setNullCheckBox.isSelected() );
				}
			}} );
		
		transferTextContentCheckBox = new JCheckBox( "Transfer text content", false );
		transferTextContentCheckBox.setToolTipText( "Will only transfer text content of source/target elements" );
		transferTextContentCheckBox.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				PropertyTransfer currentTransfer = getCurrentTransfer();
				if( currentTransfer != null )
				{
					currentTransfer.setTransferTextContent( transferTextContentCheckBox.isSelected() );
				}
			}} );
		
		ignoreEmptyCheckBox = new JCheckBox( "Ignore empty/missing values", false );
		ignoreEmptyCheckBox.setToolTipText( "Will not transfer empty or missing values" );
		ignoreEmptyCheckBox.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				PropertyTransfer currentTransfer = getCurrentTransfer();
				if( currentTransfer != null )
				{
					currentTransfer.setIgnoreEmpty( ignoreEmptyCheckBox.isSelected() );
				}
			}} );
		
		transferAllCheckBox = new JCheckBox( "Transfer to all", false );
		transferAllCheckBox.setToolTipText( "Will transfer to all matching target selections" );
		transferAllCheckBox.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				PropertyTransfer currentTransfer = getCurrentTransfer();
				if( currentTransfer != null )
				{
					currentTransfer.setTransferToAll( transferAllCheckBox.isSelected() );
				}
			}} );
		
		
		panel.add( failTransferCheckBox );
		panel.add( setNullCheckBox );
		panel.add( transferTextContentCheckBox );
		panel.add( ignoreEmptyCheckBox );
		panel.add( transferAllCheckBox );
		panel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
		return panel;
	}

	protected JXToolBar createTargetToolbar()
	{
		JXToolBar toolbar;
		toolbar = UISupport.createToolbar();
		toolbar.addSpace( 3 );
		toolbar.addFixed( new JLabel( "<html><b>Target:</b></html>"));
		toolbar.addGlue();
		
		targetStepCombo.setSelectedItem( null );
		targetStepCombo.setToolTipText( "The step the value will be transferred to" );
		targetStepCombo.setEnabled( false );
		targetStepCombo.addItemListener( 
				new StepComboItemListener( targetPropertyCombo, targetStepPropertiesListener ) );
		targetStepCombo.addItemListener( new ItemListener() {

			public void itemStateChanged(ItemEvent e)
			{
				if( e.getStateChange() == ItemEvent.SELECTED && !selecting )
				{
					String targetStep = (String) targetStepCombo.getSelectedItem();
					PropertyTransfer valueTransfer = getCurrentTransfer();
					
					if( valueTransfer != null )
					{
						valueTransfer.setTargetStepName( targetStep );
					}
				}
			}} );
		
		toolbar.addFixed( new JLabel( "Step:"));
		toolbar.addRelatedGap();
		toolbar.add( targetStepCombo );
		toolbar.addRelatedGap();
		
		toolbar.addFixed( new JLabel( "Property:"));
		toolbar.addRelatedGap();
		
		targetPropertyCombo.setToolTipText( "The property the value will be transferred to" );
		targetPropertyCombo.setEnabled( false );
		targetPropertyCombo.addItemListener( new ItemListener() {

			public void itemStateChanged(ItemEvent e)
			{
				if( e.getStateChange() == ItemEvent.SELECTED && !selecting )
				{
					String targetProperty = (String) targetPropertyCombo.getSelectedItem();
					PropertyTransfer valueTransfer = getCurrentTransfer();
					
					if( valueTransfer != null )
					{
						valueTransfer.setTargetPropertyName( targetProperty );
					}
				}
			}} );
		
		toolbar.add( targetPropertyCombo);
		return toolbar;
	}

	protected JXToolBar createSourceToolbar()
	{
		JXToolBar toolbar = UISupport.createToolbar();
		toolbar.addSpace( 3 );
		toolbar.addFixed( new JLabel( "<html><b>Source:</b></html>"));
		toolbar.addGlue();

		sourcePropertyCombo = new JComboBox();
		sourceStepModel = new DefaultComboBoxModel();
		sourceStepCombo = new JComboBox( sourceStepModel );
		sourceTestStepNameListener = new TestStepNameListener( sourceStepModel );
		
		componentEnabler.add( sourcePropertyCombo );
		componentEnabler.add( sourceStepCombo );
		
		targetPropertyCombo = new JComboBox();
		targetStepModel = new DefaultComboBoxModel();
		targetStepCombo = new JComboBox( targetStepModel );
		targetStepNameListener = new TestStepNameListener( targetStepModel );
		
		componentEnabler.add( targetPropertyCombo );
		componentEnabler.add( targetStepCombo );
		
		sourceStepPropertiesListener = new TestStepPropertiesListener( sourcePropertyCombo );
		targetStepPropertiesListener = new TestStepPropertiesListener( targetPropertyCombo );
		
		for( int c = 0; c < transferStep.getTestCase().getTestStepCount(); c++ )
		{
			WsdlTestStep testStep = (WsdlTestStep) transferStep.getTestCase().getTestStepAt( c );
			if( testStep == transferStep )
				continue;
			
			testStep.addPropertyChangeListener( TestStep.NAME_PROPERTY, sourceTestStepNameListener);
			testStep.addPropertyChangeListener( TestStep.NAME_PROPERTY, targetStepNameListener);
			
			String nm = testStep.getName();
			sourceStepModel.addElement( nm );
			targetStepModel.addElement( nm );
		}
		
		sourceStepCombo.setSelectedItem( null );
		sourceStepCombo.setToolTipText( "The step the value will be transferred from" );
		sourceStepCombo.setEnabled( false );
		sourceStepCombo.addItemListener( 
				new StepComboItemListener( sourcePropertyCombo, sourceStepPropertiesListener ) );
		sourceStepCombo.addItemListener( new ItemListener() {

			public void itemStateChanged(ItemEvent e)
			{
				if( e.getStateChange() == ItemEvent.SELECTED && !selecting )
				{
					String sourceStep = (String) sourceStepCombo.getSelectedItem();
					PropertyTransfer valueTransfer = getCurrentTransfer();
					
					if( valueTransfer != null )
					{
						valueTransfer.setSourceStepName( sourceStep );
					}
				}
			}} );
		
		toolbar.addFixed( new JLabel( "Step:"));
		toolbar.addRelatedGap();
		toolbar.add( sourceStepCombo );
		toolbar.addRelatedGap();
		
		toolbar.addFixed( new JLabel( "Property:"));
		toolbar.addRelatedGap();
		
		sourcePropertyCombo.setToolTipText( "The property the value will be transferred from" );
		sourcePropertyCombo.setEnabled( false );
		sourcePropertyCombo.addItemListener( new ItemListener() {

			public void itemStateChanged(ItemEvent e)
			{
				if( e.getStateChange() == ItemEvent.SELECTED && !selecting )
				{
					String sourceProperty = (String) sourcePropertyCombo.getSelectedItem();
					PropertyTransfer valueTransfer = getCurrentTransfer();
					
					if( valueTransfer != null )
					{
						valueTransfer.setSourcePropertyName( sourceProperty );
					}
				}
			}} );
		
		toolbar.add( sourcePropertyCombo);
		return toolbar;
	}
	
	public PropertyTransfer getCurrentTransfer()
	{
		int ix = transferList.getSelectedIndex();
		return ix == -1 ? null : transferStep.getTransferAt( ix );
	}

	/**
	 * Listen for testStep property changes and update properties combo accordingly
	 */
	
	private static final class TestStepPropertiesListener implements WsdlTestStepListener
	{
		private final JComboBox combo;

		public TestStepPropertiesListener(JComboBox combo)
		{
			this.combo = combo;
		}

		public void propertyAdded(String name)
		{
			combo.addItem( name );
			combo.setEnabled( true );
		}

		public void propertyRemoved(String name)
		{
			if( combo.getSelectedItem().equals( name ))
				combo.setSelectedItem( null );
			
			combo.removeItem( name );
			combo.setEnabled( combo.getItemCount() > 0 );
		}

		public void propertyRenamed(String oldName, String newName)
		{
			DefaultComboBoxModel model = (DefaultComboBoxModel) combo.getModel();
			
			int stepIndex = model.getIndexOf( oldName );
			if( stepIndex != -1 )
			{
				Object selectedItem = model.getSelectedItem();
				
				model.removeElementAt( stepIndex );
				model.insertElementAt( newName, stepIndex );
				
				// ensure selection
				if( selectedItem.equals( oldName ) )
					model.setSelectedItem( newName );
			}
		}

		public void propertyValueChanged(String name, String oldValue, String newValue)
		{
		}
	}

	/**
	 * Listen for teststep changes and update source/target step combos accordingly
	 */
	
	private final class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		public void testStepAdded(TestStep testStep, int index)
		{
			if( testStep.getTestCase() == transferStep.getTestCase() )
			{
				sourceStepModel.addElement( testStep.getName() );
				targetStepModel.addElement( testStep.getName() );
				
				testStep.addPropertyChangeListener( TestStep.NAME_PROPERTY, sourceTestStepNameListener);
				testStep.addPropertyChangeListener( TestStep.NAME_PROPERTY, targetStepNameListener);
			}
		}

		public void testStepMoved(TestStep testStep, int fromIndex, int offset)
		{
			if( testStep.getTestCase() == transferStep.getTestCase() )
			{
				String testStepName = testStep.getName();
				if( sourceStepModel.getIndexOf( testStepName ) == fromIndex )
				{
					String sourceStep = ( String ) sourceStepCombo.getSelectedItem();
					String sourceProperty = ( String ) sourcePropertyCombo.getSelectedItem();
					
					sourceStepModel.removeElementAt( fromIndex );
					if( fromIndex + offset > sourceStepModel.getSize() )
						sourceStepModel.addElement( testStepName );
					else 
						sourceStepModel.insertElementAt( testStepName, fromIndex + offset );
					
					sourceStepCombo.setSelectedItem( sourceStep );
					sourcePropertyCombo.setSelectedItem( sourceProperty );
				}
				
				if( targetStepModel.getIndexOf( testStepName ) == fromIndex )
				{
					String targetStep = ( String ) targetStepCombo.getSelectedItem();
					String targetProperty = ( String ) targetPropertyCombo.getSelectedItem();
					
					targetStepModel.removeElementAt( fromIndex );
					if( fromIndex + offset > targetStepModel.getSize() )
						targetStepModel.addElement( testStepName );
					else 
						targetStepModel.insertElementAt( testStepName, fromIndex + offset );
					
					targetStepCombo.setSelectedItem( targetStep );
					targetPropertyCombo.setSelectedItem( targetProperty );
				}
			}
		}

		public void testStepRemoved(TestStep testStep, int index)
		{
			if( testStep.getTestCase() == transferStep.getTestCase() )
			{
				sourceStepModel.removeElement( testStep.getName() );
				targetStepModel.removeElement( testStep.getName() );
				
				testStep.removePropertyChangeListener( TestStep.NAME_PROPERTY, sourceTestStepNameListener);
				testStep.removePropertyChangeListener( TestStep.NAME_PROPERTY, targetStepNameListener);
			}
		}
	}

	/**
	 * Listen for testStep name changes and modify comboBox model accordingly
	 */
	
	private final class TestStepNameListener implements PropertyChangeListener
	{
		private final DefaultComboBoxModel model;

		public TestStepNameListener(DefaultComboBoxModel model)
		{
			this.model = model;
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			Object oldItem = evt.getOldValue();
			int stepIndex = model.getIndexOf( oldItem );
			if( stepIndex != -1 )
			{
				Object selectedItem = model.getSelectedItem();
				Object newItem = evt.getNewValue();

				selecting = true;
				model.removeElementAt( stepIndex );
				model.insertElementAt( newItem, stepIndex );
				selecting = false;
				
				// ensure selection
				if( selectedItem.equals( oldItem ) )
					model.setSelectedItem( newItem );
			}
		}
	}

	/**
	 * Listen to step selections and update properties combo accordingly 
	 */
	
	private final class StepComboItemListener implements ItemListener
	{
		private final JComboBox propertyCombo;
		private final TestStepPropertiesListener testStepPropertiesListener;
		
		public StepComboItemListener(final JComboBox propertyCombo, TestStepPropertiesListener testStepPropertiesListener)
		{
			this.propertyCombo = propertyCombo;
			this.testStepPropertiesListener = testStepPropertiesListener;
		}

		public void itemStateChanged(ItemEvent e)
		{
			if( e.getStateChange() == ItemEvent.DESELECTED )
			{
				WsdlTestStep step = (WsdlTestStep) transferStep.getTestCase().getTestStepByName( (String) e.getItem() );
				if( step != null )
					step.removeTestStepListener( testStepPropertiesListener );
			}
			
			if( e.getStateChange() == ItemEvent.SELECTED )
			{
				String selectedItem = (String) e.getItem();
				WsdlTestStep step = (WsdlTestStep) transferStep.getTestCase().getTestStepByName( selectedItem );
				
				String[] propertyNames = step == null ? new String[0] : step.getPropertyNames();
				
				// remove read-only properties from target property
				if( propertyCombo == targetPropertyCombo )
				{
					List<String> names = new ArrayList<String>();
					for( String name : propertyNames )
					{
						TestStepProperty property = step.getProperty( name );
						if( !property.isReadOnly() )
							names.add( property.getName() );
					}
					
					propertyNames = names.toArray( new String[names.size()] );
				}
				
				propertyCombo.setModel( new DefaultComboBoxModel( propertyNames ));
				propertyCombo.setEnabled( propertyNames.length > 0 );
				
				if( propertyCombo == targetPropertyCombo )
					propertyCombo.setSelectedItem( getCurrentTransfer().getTargetPropertyName() );
				else
					propertyCombo.setSelectedItem( getCurrentTransfer().getSourcePropertyName() );
				
				step.addTestStepListener( testStepPropertiesListener );
			}
			else 
			{
				propertyCombo.removeAllItems();
				propertyCombo.setEnabled( false );
			}
		}
	}

	/**
	 * Handle updates to source path
	 */
	
	private final class SourceAreaDocumentListener extends DocumentListenerAdapter
	{
		public void update(Document document)
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferStep.getTransferAt( ix ).setSourcePath( sourceArea.getText() );
			}
		}
	}
	
	/**
	 * Handle updates to target path
	 */
	
	private final class TargetAreaDocumentListener extends DocumentListenerAdapter
	{
		public void update( Document document )
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferStep.getTransferAt( ix ).setTargetPath( targetArea.getText() );
			}
		}
	}

	/**
	 * Listen to selection changes in transfer list and update controls accordingly
	 */
	
	private final class TransferListSelectionListener implements ListSelectionListener
	{
		private PropertyTransfer transfer;

		public void valueChanged(ListSelectionEvent e)
		{
			selecting = true;
			
			if( transfer != null )
			{
				transfer.removePropertyChangeListener( transferPropertyChangeListener );
			}
			
			transfer = getCurrentTransfer();
			
			if( transfer == null )
			{
				sourceArea.setText( "" );
				targetArea.setText( "" );
				
				sourcePropertyCombo.removeAllItems();
				targetPropertyCombo.removeAllItems();
				
				sourceStepCombo.setSelectedIndex( -1 );
				targetStepCombo.setSelectedIndex( -1 );
			}
			else
			{
				transfer.addPropertyChangeListener( transferPropertyChangeListener );
				
				sourceArea.setText( transfer.getSourcePath() );
				targetArea.setText( transfer.getTargetPath() );

				sourceStepCombo.setSelectedItem( transfer.getSourceStepName() );
				sourcePropertyCombo.setSelectedItem( transfer.getSourcePropertyName() );
				
				targetStepCombo.setSelectedItem( transfer.getTargetStepName() );
				targetPropertyCombo.setSelectedItem( transfer.getTargetPropertyName() );
				
				failTransferCheckBox.setSelected( transfer.getFailOnError() );
				setNullCheckBox.setSelected( transfer.getSetNullOnMissingSource() );
				transferTextContentCheckBox.setSelected( transfer.getTransferTextContent() );
				ignoreEmptyCheckBox.setSelected( transfer.getIgnoreEmpty() );
				transferAllCheckBox.setSelected( transfer.getTransferToAll() );
			}
			
			copyButton.setEnabled( transfer != null );
			renameButton.setEnabled( transfer != null );
			deleteButton.setEnabled( transfer != null );
			declareButton.setEnabled( transfer != null );
			sourceStepCombo.setEnabled( transfer != null );
			targetStepCombo.setEnabled( transfer != null );
			sourceArea.setEnabled( transfer != null );
			targetArea.setEnabled( transfer != null );
			failTransferCheckBox.setEnabled( transfer != null );
			setNullCheckBox.setEnabled( transfer != null );
			transferTextContentCheckBox.setEnabled( transfer != null );
			ignoreEmptyCheckBox.setEnabled( transfer != null );
			transferAllCheckBox.setEnabled( transfer != null );
			
			runButton.setEnabled( transferList.getModel().getSize() > 0 );

			sourcePropertyCombo.setEnabled( transfer != null );
			targetPropertyCombo.setEnabled( transfer != null );
			
			selecting = false;
		}
	}
	
	/**
	 * Listen to property changes and update UI objects. These may have been triggered by UI so first check
	 * for actual difference so we dont end up in loop.
	 */

	private class TransferPropertyChangeListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			Object newValue = evt.getNewValue();
			
			if( evt.getPropertyName().equals( PropertyTransfer.SOURCE_PATH_PROPERTY ))
			{
				if( !sourceArea.getText().equals( newValue ))
					sourceArea.setText( (String) newValue );
			}
			else if( evt.getPropertyName().equals( PropertyTransfer.TARGET_PATH_PROPERTY ))
			{
				if( !targetArea.getText().equals( newValue ))
					targetArea.setText( (String) newValue );
			}
			else if( evt.getPropertyName().equals( PropertyTransfer.SOURCE_STEP_PROPERTY ))
			{
				Object selectedItem = sourceStepCombo.getSelectedItem();
				if( newValue == null || selectedItem == null || !selectedItem.equals( newValue ))
				{
					selecting = true;
					sourceStepCombo.setSelectedItem( newValue );
					selecting = false;
				}
			}
			else if( evt.getPropertyName().equals( PropertyTransfer.TARGET_STEP_PROPERTY ))
			{
				Object selectedItem = targetStepCombo.getSelectedItem();
				if( newValue == null || selectedItem == null || !selectedItem.equals( newValue ))
				{
					selecting = true;
					targetStepCombo.setSelectedItem( newValue );
					selecting = false;
				}
			}
			else if( evt.getPropertyName().equals( PropertyTransfer.SOURCE_TYPE_PROPERTY ))
			{
				Object selectedItem = sourcePropertyCombo.getSelectedItem();
				if( selectedItem == null || !selectedItem.equals( newValue ))
					sourcePropertyCombo.setSelectedItem( newValue );
			}
			else if( evt.getPropertyName().equals( PropertyTransfer.TARGET_TYPE_PROPERTY ))
			{
				Object selectedItem = targetPropertyCombo.getSelectedItem();
				if( selectedItem == null || !selectedItem.equals( newValue ))
					targetPropertyCombo.setSelectedItem( newValue );
			}
		}
	}
	
	private final class AddAction extends AbstractAction
	{
		public AddAction()
		{
			super( "Add" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String name = UISupport.prompt( "Specify name for value transfer", "Add Transfer", "" );
			if( name == null || name.trim().length() == 0 ) return;
			
			transferStep.addTransfer( name );
			
			listModel.addElement( name );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class CopyAction extends AbstractAction
	{
		public CopyAction()
		{
			super( "Copy" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			int ix = transferList.getSelectedIndex();
			PropertyTransfer config = transferStep.getTransferAt( ix );
			
			String name = UISupport.prompt( "Specify name for value transfer", "Copy Transfer", config.getName() );
			if( name == null || name.trim().length() == 0 ) return;
			
			PropertyTransfer transfer = transferStep.addTransfer( name );
			transfer.setSourceStepName( config.getSourceStepName() );
			transfer.setSourcePropertyName( config.getSourcePropertyName() );
			transfer.setSourcePath( config.getSourcePath() );
			transfer.setTargetStepName( config.getTargetStepName() );
			transfer.setTargetPropertyName( config.getTargetPropertyName() );
			transfer.setTargetPath( config.getTargetPath() );
			
			listModel.addElement( name );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super( "Delete" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( UISupport.confirm( "Delete selected transfer", "Delete Transfer" )) 
			{
				transferList.setSelectedIndex( -1 );
				
				int ix = transferList.getSelectedIndex();
				transferStep.removeTransferAt( ix );
				listModel.remove( ix );
				
				if( listModel.getSize() > 0 )
				{
					transferList.setSelectedIndex( ix > listModel.getSize()-1 ? listModel.getSize()-1 : ix );
				}
			}
		}
	}
	
	private final class RenameAction extends AbstractAction
	{
		public RenameAction()
		{
			super( "Rename" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			PropertyTransfer transfer = getCurrentTransfer();
			
			String newName = UISupport.prompt( "Specify new name for transfer", "Rename Transfer", transfer.getName() );
			
			if( !transfer.getName().equals( newName )) 
			{
				listModel.setElementAt( newName, transferList.getSelectedIndex() );
				transfer.setName( newName );
			}
		}
	}
	
	
	private final class DeclareNamespacesAction extends AbstractAction
	{
		public DeclareNamespacesAction()
		{
			super( "Declare" );
			putValue( Action.SHORT_DESCRIPTION, "Declare available response/request namespaces in source/target expressions" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			try
			{
				TestStep previousStep = getCurrentTransfer().getSourceStep();
				
				if ( previousStep instanceof WsdlTestRequestStep )
				{
					WsdlTestRequest testRequest = ((WsdlTestRequestStep)previousStep).getTestRequest();
					sourceArea.setText( XmlUtils.declareXPathNamespaces( testRequest.getOperation().getInterface() ) + sourceArea.getText() );
				}
				else UISupport.showErrorMessage( "Source step is not a request" );

				TestStep nextStep = getCurrentTransfer().getTargetStep();
				
				if (nextStep instanceof WsdlTestRequestStep)
				{
					WsdlTestRequest testRequest = ((WsdlTestRequestStep)nextStep).getTestRequest();
					targetArea.setText( XmlUtils.declareXPathNamespaces( testRequest.getOperation().getInterface()) + targetArea.getText() );
				}
				else UISupport.showErrorMessage( "Target step is not a request" );
			}
			catch (Exception e1)
			{
				UISupport.showErrorMessage( e1 );
				e1.printStackTrace();
			}
		}
	}
	
	private final class RunAction extends AbstractAction
	{
		public RunAction()
		{
			super( "Run" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( listModel.getSize() == 0 )
			{
				UISupport.showErrorMessage( "Missing transfers!" );
				return;
			}
			
			WsdlTestRunContext context = new WsdlTestRunContext( transferStep );
			ValueTransferResult result = (ValueTransferResult) transferStep.run( null, context );
			
			if( result.getStatus() == TestStepResult.TestStepStatus.FAILED )
			{
				StringBuffer msg = new StringBuffer();
				String[] messages = result.getMessages();
				for( int c = 0; c < result.getTransferCount() && c < messages.length; c++ )
				{
					msg.append( messages[c] ).append( ':' ).append( Arrays.toString( result.getTransferredValuesAt( c ))).append( "\r\n" );
				}
				
				UISupport.showErrorMessage( msg.toString() );
			}
			else 
			{
				UISupport.showInfoMessage( "Transfers completed with status " + result.getStatus() );
			}
		}
	}
	
	private final class CloseAction extends AbstractAction
	{
		public CloseAction()
		{
			super( "Close" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			SoapUI.getDesktop().closeDesktopPanel( transferStep );
		}
	}

	public ModelItem getModelItem()
	{
		return transferStep;
	}

	public boolean onClose( boolean canCancel )
	{
		transferStep.getTestCase().getTestSuite().removeTestSuiteListener( testSuiteListener ); 
		
		PropertyTransfer transfer = getCurrentTransfer();
		
		if( transfer != null )
		{
			transfer.removePropertyChangeListener( transferPropertyChangeListener );
		}
		
		for( int c = 0; c < transferStep.getTestCase().getTestStepCount(); c++ )
		{
			WsdlTestStep testStep = (WsdlTestStep) transferStep.getTestCase().getTestStepAt( c );
			
			testStep.removePropertyChangeListener( TestStep.NAME_PROPERTY, sourceTestStepNameListener);
			testStep.removePropertyChangeListener( TestStep.NAME_PROPERTY, targetStepNameListener);
		}
		
		Object item = sourceStepCombo.getSelectedItem();
		if( item != null )
		{
			WsdlTestStep step = (WsdlTestStep) transferStep.getTestCase().getTestStepByName( (String) item );
			if( step != null )
				step.removeTestStepListener( sourceStepPropertiesListener );
		}

		item = targetStepCombo.getSelectedItem();
		if( item != null )
		{
			WsdlTestStep step = (WsdlTestStep) transferStep.getTestCase().getTestStepByName( (String) item );
			if( step != null )
				step.removeTestStepListener( targetStepPropertiesListener );
		}

		componentEnabler.release();
		
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == transferStep || modelItem == transferStep.getTestCase() ||
				modelItem == transferStep.getTestCase().getTestSuite() ||
				modelItem == transferStep.getTestCase().getTestSuite().getProject();
	}

	public String getTitle()
	{
		return transferStep.getTestCase().getName() + " - " +  transferStep.getName();
	}
	
	public String getDescription()
	{
		return "Property Transfer: [" + transferStep.getName() + "] - " + transferStep.getTestStepTitle();
	}

	public Icon getIcon()
	{
		return getModelItem().getIcon();
	}

	public boolean selectTransfer(PropertyTransfer transfer)
	{
		for( int c = 0; c < transferStep.getTransferCount(); c++ )
		{
			if( transferStep.getTransferAt( c ) == transfer )
			{
				transferList.setSelectedIndex( c );
				return true;
			}
		}
		
		return false;
	}
}
