/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.auth.CredentialsProvider;

import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.impl.wsdl.submit.transports.http.TimeablePostMethod;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;

/**
 * RequestFilter for setting preemptive authentication and related credentials 
 */

public class HttpAuthenticationRequestFilter implements RequestFilter
{
	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		String username = wsdlRequest.getUsername();
		
		// check for authorization prerequisites
		if( username == null || username.length() == 0 )
			return;
		
		HttpClient httpClient = (HttpClient) context.getProperty( BaseHttpRequestTransport.HTTP_CLIENT );
		Settings settings = wsdlRequest.getSettings();
		String password = wsdlRequest.getPassword();
		
		//	 set preemptive authentication
		if (settings.getBoolean(HttpSettings.AUTHENTICATE_PREEMPTIVELY))
		{
			httpClient.getParams().setAuthenticationPreemptive(true);
			HttpState state = (HttpState) context.getProperty( SubmitContext.HTTP_STATE_PROPERTY );
			
			if( state != null )
			{
				Credentials defaultcreds = new UsernamePasswordCredentials(username, password);
				state.setCredentials(AuthScope.ANY, defaultcreds);
			}
		}
		else
		{
			httpClient.getParams().setAuthenticationPreemptive(false);
		}
		
		TimeablePostMethod postMethod = (TimeablePostMethod) context.getProperty( BaseHttpRequestTransport.POST_METHOD );
		
		postMethod.getParams().setParameter(CredentialsProvider.PROVIDER, new WsdlRequestCredentialsProvider( wsdlRequest ));
		postMethod.setDoAuthentication(true);
	}
}
