/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import org.apache.commons.httpclient.URI;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.impl.wsdl.submit.transports.http.TimeablePostMethod;
import com.eviware.soapui.impl.wsdl.support.soap.SoapVersion;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.settings.HttpSettings;

/**
 * RequestFilter that adds SOAP 1.1 specific headers
 * 
 * @author Ole.Matzura
 */

public class SoapHeadersRequestFilter implements RequestFilter
{
	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		TimeablePostMethod postMethod = (TimeablePostMethod) context.getProperty( BaseHttpRequestTransport.POST_METHOD );
		
		WsdlInterface wsdlInterface = (WsdlInterface) wsdlRequest.getOperation().getInterface();
		String strURL = wsdlInterface.getEndpointURL(wsdlRequest.getEndpoint());
		strURL = PropertyExpansionRequestFilter.expandProperties( context, strURL );
		try
		{
			postMethod.setURI(new URI(strURL, wsdlRequest.getSettings().getBoolean( HttpSettings.ENCODED_URLS )));
		}
		catch ( Exception e)
		{
			e.printStackTrace();
		}
		
		//	 init content-type and encoding
		String encoding = wsdlRequest.getEncoding();
		
		SoapVersion soapVersion = wsdlInterface.getSoapVersion();
		postMethod.setRequestHeader( "Content-Type", soapVersion.getContentTypeHttpHeader( encoding ));

		// set soapaction
		String soapAction = wsdlRequest.getOperation().getAction();
		if (soapAction == null || soapAction.length() == 0)
		{
			soapAction = "\"\"";
		}
		else
		{
			soapAction = "\"" + soapAction + "\"";
		}

		postMethod.setRequestHeader("SOAPAction", soapAction);

	}
}
