package com.eviware.soapui.impl.wsdl.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.AssertionsListener;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.WsdlAssertionRegistry;

public class AssertionsSupport implements PropertyChangeListener
{
	private List<AssertionsListener> assertionsListeners = new ArrayList<AssertionsListener>();
	private List<WsdlMessageAssertion> assertions = new ArrayList<WsdlMessageAssertion>();
	private final Assertable assertable;

	public AssertionsSupport( Assertable assertable, List<RequestAssertionConfig> assertionList )
	{
		this.assertable = assertable;

		for( RequestAssertionConfig rac : assertionList )
		{
			addWsdlAssertion( rac );
		}
	}

	public WsdlMessageAssertion addWsdlAssertion( RequestAssertionConfig config )
	{
		try
		{
			WsdlMessageAssertion assertion = WsdlAssertionRegistry.getInstance().buildAssertion(
						config, assertable );
			assertions.add( assertion );
			assertion.addPropertyChangeListener( this );

			return assertion;
		}
		catch( Exception e )
		{
			e.printStackTrace();
			return null;
		}
	}

	public void propertyChange( PropertyChangeEvent arg0 )
	{
		if( assertable instanceof PropertyChangeListener )
			((PropertyChangeListener)assertable).propertyChange( arg0 );
	}

	public int getAssertionCount()
	{
		return assertions.size();
	}

	public WsdlMessageAssertion getAssertionAt( int c )
	{
		return assertions.get( c );
	}

	public void addAssertionsListener( AssertionsListener listener )
	{
		assertionsListeners.add( listener );
	}

	public void removeAssertionsListener( AssertionsListener listener )
	{
		assertionsListeners.remove( listener );
	}

	public int removeAssertion( WsdlMessageAssertion assertion )
	{
		int ix = assertions.indexOf( assertion );
		if( ix == -1 )
		{
			throw new RuntimeException( "assertion [" + assertion.getName() + "] not available " );
		}

		assertion.removePropertyChangeListener( this );
		assertions.remove( ix );
		fireAssertionRemoved( assertion );

		return ix;
	}

	public void release()
	{
		for( WsdlMessageAssertion assertion : assertions )
			assertion.release();

	}

	public Iterator<WsdlMessageAssertion> iterator()
	{
		return assertions.iterator();
	}

	public void fireAssertionAdded( WsdlMessageAssertion assertion )
	{
		AssertionsListener[] listeners = assertionsListeners
					.toArray( new AssertionsListener[assertionsListeners.size()] );

		for( int c = 0; c < listeners.length; c++ )
		{
			listeners[c].assertionAdded( assertion );
		}
	}

	public void fireAssertionRemoved( WsdlMessageAssertion assertion )
	{
		AssertionsListener[] listeners = assertionsListeners
					.toArray( new AssertionsListener[assertionsListeners.size()] );

		for( int c = 0; c < listeners.length; c++ )
		{
			listeners[c].assertionRemoved( assertion );
		}
	}

	public void updateConfig( List<RequestAssertionConfig> assertionList )
	{
      for (int i = 0; i < assertionList.size(); i++)
      {
         RequestAssertionConfig config = assertionList.get( i );
         assertions.get( i ).updateConfig( config );
      }
	}
}
