 /*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */
 
package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStepResult;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepResult;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JPropertiesTable;
import com.eviware.soapui.support.components.JPropertiesTable.PropertyFormatter;
import com.eviware.soapui.support.xml.JXEditTextArea;
import com.eviware.soapui.support.xml.XmlUtils;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.support.DefaultDesktopPanel;

/**
 * Shows a desktop-panel with the TestStepResult for a WsdlTestRequestStepResult
 * 
 * @author Ole.Matzura
 */

public class ShowRequestStepResultsAction extends AbstractAction
{
	private final WsdlTestRequestStepResult result;
	private DefaultDesktopPanel desktopPanel;

	public ShowRequestStepResultsAction(WsdlTestStepResult result)
	{
		super( "Show Results" );
		
		this.result = (WsdlTestRequestStepResult) result;
	}

	public void actionPerformed(ActionEvent e)
	{
		try
		{
			if( result.isDiscarded() )
				UISupport.showInfoMessage( "Request has been discarded.." );
			else
				UISupport.showDesktopPanel(buildFrame());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}		
	}

	private DesktopPanel buildFrame()
	{
		if( desktopPanel == null )
		{
			desktopPanel = new DefaultDesktopPanel( "TestStep Result", 
						"TestStep result for " + result.getTestStep().getName(), buildContent() );
		}
		
		return desktopPanel;
	}

	private JComponent buildContent()
	{
		JTabbedPane messageTabs = new JTabbedPane();
		messageTabs.addTab( "Request Properties", buildPropertiesTab() );
		messageTabs.addTab( "Request Message", buildRequestTab() );
		messageTabs.addTab( "Response Message", buildResponseTab() );
		messageTabs.setPreferredSize( new Dimension( 500, 400 ));
		
		JPanel panel = new JPanel( new BorderLayout() );
		panel.add( UISupport.createTabPanel( messageTabs, true ), BorderLayout.CENTER );

		return panel;
	}

	private Component buildPropertiesTab()
	{
		JPropertiesTable table = new JPropertiesTable<WsdlTestRequestStepResult>( "Result Properties", result );
		table.addProperty( "Status", "status" );
		table.addProperty( "Time Taken", "timeTaken" );
		table.addProperty( "Size", "size" );
		table.addProperty( "Timestamp", "timeStamp", false, new TimestampPropertyFormatter());
		table.addProperty( "TestStep", "testStep", false, new TestStepPropertyFormatter());
		
		table.addProperty( "Encoding", "encoding" );
   	table.addProperty( "Endpoint", "endpoint" );
   	table.addProperty( "Username", "username" );
   	table.addProperty( "Password", "password" );
   	table.addProperty( "Domain", "domain" );
   	
   	DefaultListModel listModel = new DefaultListModel();
		JList messageList = new JList( listModel );
		String[] messages = result.getMessages();
		for( String message : messages )
		{
			listModel.addElement( message );
		}
		
		JSplitPane split = UISupport.createVerticalSplit( table, new JScrollPane( messageList ));
		split.setDividerLocation( 200 );
		return split;
	}

	private Component buildResponseTab()
	{
		JXEditTextArea resultArea = JXEditTextArea.createXmlEditor();
      Response response = result.getResponse();
      if( response != null )
      	resultArea.setText( XmlUtils.prettyPrintXml( response.getContentAsString() ) );
      else
      	resultArea.setText( "- null -" );
      resultArea.setEditable( false );
      resultArea.setToolTipText( "Response Content" );
		JScrollPane scrollPane = new JScrollPane( resultArea );
		return scrollPane;
	}

	private Component buildRequestTab()
	{
		JXEditTextArea resultArea = JXEditTextArea.createXmlEditor();
      String requestContent = result.getRequestContent();
      if( requestContent != null )
      	resultArea.setText( XmlUtils.prettyPrintXml( requestContent ) );
      else 
      	resultArea.setText( "- null -" );
      resultArea.setEditable( false );
      resultArea.setToolTipText( "Request Content" );
		JScrollPane scrollPane = new JScrollPane( resultArea );
		return scrollPane;
	}
	
	private static final class TestStepPropertyFormatter implements PropertyFormatter
	{
		public Object format(String propertyName, Object value)
		{
			return ((TestStep)value).getName() ;
		}
	}

	private static final class TimestampPropertyFormatter implements PropertyFormatter
	{
		public Object format(String propertyName, Object value)
		{
			return new Date( Long.valueOf( value.toString() ) );
		}
	}
}
