/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.report;

import java.io.File;
import java.io.FileWriter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable.AssertionStatus;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlMessageAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.testsuite.TestRunner.Status;

public class JUnitReportCollector implements TestRunListener {

	HashMap<String, JUnitReport> reports;
	HashMap<TestCase, StringBuffer> failures;
	
	public JUnitReportCollector() {
		reports = new HashMap<String, JUnitReport>();
		failures = new HashMap<TestCase, StringBuffer>();
	}
	
	public void saveReports(String path) throws Exception {
		Iterator<String> keyset = reports.keySet().iterator();
		while (keyset.hasNext()) {
			String name = keyset.next();
			JUnitReport report = reports.get(name);
			saveReport(report, path + File.separatorChar + "TEST-" + name + ".xml");
		}
	}
	
	private void saveReport(JUnitReport report, String filename) throws Exception {
		FileWriter fileWriter = new FileWriter(filename);
		fileWriter.write(report.toString());
		fileWriter.close();
	}
	
	public String getReport() {
		Set<String> keys = reports.keySet();
		if (keys.size() > 0) {
			String key = (String)keys.toArray()[0];
			return reports.get(key).toString();
		}
		return "No reports..:";
	}
	
	public void afterRun(TestRunner testRunner, TestRunContext runContext) {
		TestCase testCase = testRunner.getTestCase();
		JUnitReport report = reports.get(testCase.getTestSuite().getName());
		
		if (Status.INITIALIZED != testRunner.getStatus()
				&& Status.RUNNING != testRunner.getStatus()) {
			if (Status.CANCELED == testRunner.getStatus()) {
				report.addTestCaseWithFailure(testCase.getName(), testRunner.getTimeTaken(), testRunner.getReason(), "");
			}
			if ( Status.FAILED == testRunner.getStatus()) {
				String msg = "";
				if (failures.containsKey(testCase)) {
					msg = failures.get(testCase).toString();
				}
				report.addTestCaseWithFailure(testCase.getName(), testRunner.getTimeTaken(), testRunner.getReason(), msg);
			}
			if (Status.FINISHED == testRunner.getStatus()) {
				report.addTestCase(testCase.getName(), testRunner.getTimeTaken());
			}
			
		}
	}

	public void afterStep(TestRunner testRunner, TestRunContext runContext, TestStepResult result) {
		TestStep currentStep = runContext.getCurrentStep();
		TestCase testCase = currentStep.getTestCase();
		
		if( currentStep instanceof WsdlTestRequestStep )
		{
			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) currentStep;
			for( int c = 0; c < requestStep.getAssertionCount(); c++ )
			{
				WsdlMessageAssertion assertion = requestStep.getAssertionAt( c );
				
				if( assertion.getStatus() != AssertionStatus.VALID ) {
					StringBuffer buf = new StringBuffer();
					if (failures.containsKey(testCase)) {
						buf = failures.get(testCase);
					} else
						failures.put(testCase, buf);
					
					buf.append( assertion.getName() + " in [" + requestStep.getName() + "] failed;\n" );
					buf.append( Arrays.toString( assertion.getErrors() ) + "\n" );
					buf.append( "Request: " + requestStep.getTestRequest().getRequestContent() + "\n" );
					buf.append( "Response: " + requestStep.getTestRequest().getResponse().getContentAsString() + "\n" );
				}
			}
		}
	}

	public void beforeRun(TestRunner testRunner, TestRunContext runContext) {
		TestCase testCase = testRunner.getTestCase();
		TestSuite testSuite = testCase.getTestSuite();
		if (!reports.containsKey(testSuite.getName())) {
			JUnitReport report = new JUnitReport();
			report.setTestSuiteName(testSuite.getProject().getName() + "." + testSuite.getName());
			reports.put(testSuite.getName(), report);
		}
	}

	public void beforeStep(TestRunner testRunner, TestRunContext runContext) {
		
	}


}
